/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./analyseurSyntaxique.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ASC
#define ASC

#include "analyseurSyntaxique.h"

AnalyseurSyntaxique::AnalyseurSyntaxique(AnalyseurLexical* a,int pap,QSemaphore* s):pg(a){
	pasApas = pap;
	sem = s;
  al =a;
}
AnalyseurSyntaxique::~AnalyseurSyntaxique(){
}

void AnalyseurSyntaxique::pointArret(string s){
	if(pasApas == 1){
		cout<<"Point d'arret : "<<s<<endl;
		sem->acquire();
	}
}



void AnalyseurSyntaxique::analyser(){
  if(al->ccVide()==1){
    throw("Chaine codee vide. Relancer l'analyse lexicale");
  }
  pg.reset();
  princ();
  pg.afficherPO();
}


vector<vector<int> > AnalyseurSyntaxique::progObjet(){
  return pg.programmeObjet();
}


int AnalyseurSyntaxique::horner(string m){
  int nb;
  nb=m[0]  - '0';
  unsigned i=1;
  while (i<m.size()) {
    nb=nb*10+m[i] - '0'; //schema de Horner
    ++i;
  }
  return nb;
}

/*
 *  Representations des regles de grammaire
*/

void AnalyseurSyntaxique::princ(){
  pg.genererDebut();
  lireMotCle("program");
  lireCaractere(":");
  corpsPrinc();
  lireCaractere(".");
  pg.genererFin();
}


void AnalyseurSyntaxique::corpsPrinc(){
  if(!(*al).voirMotCle("begin")){
	  partieDecla();
    }
  lireMotCle("begin");
  if(!(*al).voirMotCle("end")){
    listeInstrs();
  }
  lireMotCle("end");
}

void AnalyseurSyntaxique::partieDecla(){
	if(al->voirMotCle("integer") || (*al).voirMotCle("bool")){
		listeDecla();
	}
	if(al->voirMotCle("routine")){
		listeDeclaRoutine();
	}
}


void AnalyseurSyntaxique::listeDecla(){
  decla();
  if((*al).voirMotCle("integer") || (*al).voirMotCle("bool")){
    listeDecla();
  }
}


void AnalyseurSyntaxique::decla(){
  if((*al).voirMotCle("integer") || (*al).voirMotCle("bool")){
    string type = (*al).get();
    lire();
    listeIdent();
    pg.insererIdents(type);
    lireCaractere(";"); 
  }
}


void AnalyseurSyntaxique::listeIdent(){
  pg.newDecla();
  ident();
  while((*al).voirCaractere(",")){
    lire();
    ident();
  }
}


void AnalyseurSyntaxique::ident(){
  if((*al).voirIdentificateur() && !(*al).voirMotCle("integer") && !(*al).voirMotCle("bool")){
    pg.savIdent((*al).get());
    lire();
  }else{
    (*al).erreur(5,(*al).get());
  }
}


void AnalyseurSyntaxique::listeDeclaRoutine(){
	int coBr = pg.genererBr();
  declaRoutine();
  while((*al).voirMotCle("routine")){
    declaRoutine();
  }
	pg.brancher(coBr);
}


void AnalyseurSyntaxique::declaRoutine(){
  lire();
  string nomFonct = "";
  if((*al).voirMotCle("integer") || (*al).voirMotCle("void") || (*al).voirMotCle("bool")){
		varGlo_retour = (*al).get();
    lire();
  }else{
    (*al).erreur(11,""); // type de retour attendu
  }
  if((*al).voirIdentificateur()){
    nomFonct = (*al).get();
    lire();
  }else{
    (*al).erreur(1,""); //identificateur attendu
  }
	pg.insererRoutine(nomFonct,varGlo_retour);
  pg.TDILocale();

  lireCaractere("(");
  if(!(*al).voirCaractere(")")){
    listeParam(nomFonct);
  }
  lireCaractere(")");
  corps();
	if(varGlo_retour.compare("void")==0){
				pg.sortirRoutine(0); // sans retour
	}
	
	lireMotCle("end");
  lireCaractere(";");
	varGlo_retour = "";
	
  pg.TDIGlobale();
	
}


void AnalyseurSyntaxique::listeParam(string n){
  param(n);
  while((*al).voirCaractere(",")){
    lire();
    param(n);
  }
}


void AnalyseurSyntaxique::param(string n){
	string type;
  if((*al).voirMotCle("integer") || (*al).voirMotCle("bool")){
    type = (*al).get();
    lire();
  }else{
    (*al).erreur(1,""); //identificateur attendu
  }

  if((*al).voirIdentificateur() && !(*al).voirMotCle("integer") && !(*al).voirMotCle("bool")){
    pg.ajouterParam(n,(*al).get(),type);
    lire();
  }else{
    (*al).erreur(5,(*al).get());  //ne peut etre utilise comme nom de var
  }
}


void AnalyseurSyntaxique::listeDeclaInSub(){
  decla();
  if((*al).voirMotCle("integer") || (*al).voirMotCle("bool")){
    listeDecla();
  }else if((*al).voirMotCle("routine")){
    (*al).erreur(10,"");
  }
}


void AnalyseurSyntaxique::corps(){
  if(!(*al).voirMotCle("begin")){
    listeDeclaInSub();
  }
  lireMotCle("begin");
  if(!(*al).voirMotCle("end")){
    listeInstrs();
  }
}


void AnalyseurSyntaxique::listeInstrs(){
	pg.setFinBloc(0);
  instr();
  while((*al).voirCaractere(";")){
    lire();
    instr();
  }
	pg.setFinBloc(0);
	
}


void AnalyseurSyntaxique::instr(){
  if((*al).voirIdentificateur()){
		int varStatus = pg.estDeclaree((*al).get());
    if(varStatus == 0){
      (*al).erreur(8,(*al).get());
    }
    if(pg.estVariable((*al).get())){
      string typeG = pg.typeDe((*al).get());
			if(varStatus == 1){
				pg.genererEmpiler_ADDR(pg.addrVar((*al).get()));
			}else{
				pg.genererEmpiler_ADDR_S(pg.addrVar((*al).get()));
			}
			pg.estInit((*al).get());
      lireIdentificateur();
      if((*al).voirCaractere("=")){  //affectation
        lire();
        string typeD = expression();
        if(!pg.typage(typeG,typeD)){
          string dif = "'"+typeG+"' & '"+typeD+"'";
          (*al).erreur(7,dif);
        }
        pg.genererAffect();
			}
			}else if(pg.estRoutine(al->get())){
				
				string nom = al->get();
				int nbParam=pg.nombreParam(nom);
				lire();
				pg.genererBase();
				lireCaractere("(");
				int i=0;
				while(i<nbParam){
					if(!pg.typage(expression(),pg.typeParam(nom,i))){
						(*al).erreur(12,pg.typeParam(nom,i));
					}
					++i;
					if(!al->voirCaractere(")")){
						lireCaractere(",");
					}
				}
				if(i!=nbParam){
					(*al).erreur(13,"");
				}
				lireCaractere(")");
				pg.remplirBase(nbParam);
				pg.derouter(nom);
			}
  }else if((*al).voirMotCle("print")){
    lire();
    lireCaractere("(");
    expression();
    lireCaractere(")");
    pg.genererPrint();
  }else if((*al).voirMotCle("if")){
    alternatives();
  }else if((*al).voirMotCle("while")){
    boucleWhile();
	}else if((*al).voirMotCle("for")){
    boucleFor();
	}else if(al->voirMotCle("return")){
		lire();
		string retour = expression();
		if(varGlo_retour.compare("")==0){
			al->erreur(17,""); // Une procedure ne renvoie rien (meme pas void)
		}
		if(varGlo_retour.compare("void")==0){
			al->erreur(15,""); // Une procedure ne renvoie rien (meme pas void)
		}
		if(!pg.typage(retour,varGlo_retour)){
			al->erreur(16,varGlo_retour); //la fonction doit retourner un...
		}
		pg.sortirRoutine(1); // avec retour
		pg.setFinBloc(1);
  }else{
    (*al).erreur(3,"");
  }
}

string AnalyseurSyntaxique::expression(){
  string type = exprAnd();
  while((*al).voirMotCle("or")){
    string typeD = exprAnd();
    if(!pg.typage(type,typeD,"bool")){
      (*al).erreur(6,"bool");
    }
  }
  return type;
}


string AnalyseurSyntaxique::exprAnd(){
  string type = exprTest();
  while((*al).voirMotCle("and")){
    lireMotCle("and");
    string typeD = exprTest();
    if(pg.typage(type,typeD,"bool")){
      (*al).erreur(6,"bool");
    }
  }
  return type;
}


string AnalyseurSyntaxique::exprTest(){ //chargee des tests
  string type = exprAdd();
  if((*al).voirMotCle("==") || (*al).voirMotCle("!=")){
    string op = (*al).get();
    lire();
    string typeD = exprAdd();
    if(!pg.typage(type,typeD)){
      (*al).erreur(7,type+" "+typeD);
    }
    pg.genererTest(op);
    type ="bool";
  }else if((*al).voirMotCle("<=") || (*al).voirMotCle(">=") || (*al).voirCaractere("<") || (*al).voirCaractere(">")){
    string op = (*al).get();
    lire();
    string typeD = exprAdd();
    if(!pg.typage(type,typeD,"integer")){
      (*al).erreur(6,"integer");
    }
    pg.genererTest(op);
    type = "bool";
  }
  return type;
}


string AnalyseurSyntaxique::exprAdd(){ //chargee des additions
  string type = exprMult();
  while((*al).voirCaractere("+") || (*al).voirCaractere("-")){
  string op=(*al).get();
  lire();
  string typeD = exprMult();
  if(!pg.typage(type,typeD) || !pg.typage(type,"integer")){
    (*al).erreur(6,"integer");
  }
  pg.genererAdd(op);
  }
  return type;  
}


string AnalyseurSyntaxique::exprMult(){ //chargee des multiplications
  string type = terminal();
  while((*al).voirCaractere("*") || (*al).voirCaractere("/")){
    string op=(*al).get();
    lire();
    terminal();
    pg.genererMult(op);
  }
  return type;
}


string AnalyseurSyntaxique::terminal(){//chargee des signes unaires
  string type="";
  string signe ="";
  if((*al).voirCaractere("-") || (*al).voirCaractere("+")){
    signe = (*al).get();
    lire();
    type = elemTerminal();
    if(!pg.typage(type,"integer")){
      (*al).erreur(6,"integer");
    }
    pg.genererInverser(signe);
  }else{
    type = elemTerminal();
  }
  return type;
}


string AnalyseurSyntaxique::elemTerminal(){
  string type ="";
  if((*al).voirNombre()){
    pg.genererEmpiler(horner((*al).get()));
    lire();
    type = "integer";
  }else if((*al).voirIdentificateur()){
		if(pg.estVariable(al->get())){
			int varStatus = pg.estDeclaree((*al).get());
			if(varStatus == 0){
				(*al).erreur(8,(*al).get());
			}
			if(pg.init((*al).get()) ==0){  // non intialisee
				al->warning(0,(*al).get());
			}
			if(varStatus == 1){
				pg.genererEmpiler_ADDR(pg.addrVar((*al).get()));
			}else{
				pg.genererEmpiler_ADDR_S(pg.addrVar((*al).get()));
			}
			pg.genererValPile();
			type = pg.typeDe((*al).get());
			pg.estUtilisee((*al).get());
			lire();
		}else if(pg.estRoutine(al->get())){
			string nom = al->get();
			int nbParam=pg.nombreParam(nom);
			if(pg.typage(pg.typeDe(nom),"void")){
				al->erreur(14,nom);
			}
			lire();
			pg.genererBase();
			lireCaractere("(");
			int i=0;
			while(i<nbParam){
				if(!pg.typage(expression(),pg.typeParam(nom,i))){
					(*al).erreur(12,pg.typeParam(nom,i));
				}
				++i;
				if(!al->voirCaractere(")")){
					lireCaractere(",");
				}
			}
			if(i!=nbParam){
				(*al).erreur(13,"");
			}
			lireCaractere(")");
			pg.remplirBase(nbParam);
			pg.derouter(nom);
			return pg.typeDe(nom);
		}
  }else if((*al).voirCaractere("(")){
    lire();
    type = expression();
    lireCaractere(")");
  }else if((*al).voirMotCle("true")){
    lire();
    pg.genererEmpiler(1);
    return "bool";
  }else if((*al).voirMotCle("false")){
    lire();
    pg.genererEmpiler(0);
    return "bool";
  }
  return type;
}

void AnalyseurSyntaxique::alternatives(){
  vector<int> brs;
  lireMotCle("if");
  if(!pg.typage(expression(),"bool")){
    (*al).erreur(6,"bool");
  }
  lireMotCle("then");
  int coT = pg.genererBze();
  listeInstrs();
  //pg.brancher(coT);
	if((*al).voirMotCle("elsif") || (*al).voirMotCle("else")){
		while((*al).voirMotCle("elsif")){
			int co = pg.genererBr(coT);
			brs.push_back(co);
			lire();
			if(!pg.typage(expression(),"bool")){
				(*al).erreur(6,"bool");
			}
			lireMotCle("then");
	//    pg.brancher(coT);
			coT = pg.genererBze();
			listeInstrs();
		}
  //pg.brancher(coT);
		if((*al).voirMotCle("else")){
			lire();
			int co = pg.genererBr(coT);
			brs.push_back(co);
			listeInstrs();
		}
	}else{
		pg.brancher(coT);
	}
  vector <int>::iterator i;
  for(i=brs.begin();i!=brs.end();++i){
   pg.brancher(*i);
  }
  lireMotCle("fi");
}


void AnalyseurSyntaxique::boucleWhile(){
  lireMotCle("while");
  int co = pg.co;
  if(!pg.typage(expression(),"bool")){
    (*al).erreur(6,"bool");
  }
  lireMotCle("do");
  int bze = pg.genererBze();
  listeInstrs();
  pg.brancher(pg.genererBr(),co);
  pg.brancher(bze);  
  lireMotCle("done");
}

//for i=0;i<0;i=i+1 do

void AnalyseurSyntaxique::boucleFor(){
  lireMotCle("for");
  instr(); //INIT
	lireCaractere(";");
  int coAvantCond = pg.co;
	string type = expression(); //CONDITION ARRET
	lireCaractere(";");
  if(!pg.typage(type,"bool")){ 
    (*al).erreur(6,type);
  }
	int bze = pg.genererBze(); // branchement pour sortir
	int br0 = pg.genererBr();
	instr();  //progression
	lireMotCle("do");
	int br1 = pg.genererBr();
	pg.brancher(br1,coAvantCond);
	pg.brancher(br0);
  listeInstrs();  //corps de la boucle
	int br2 = pg.genererBr();
	pg.brancher(br2,br0+1);
	lireMotCle("done");
	pg.brancher(bze);
}

void AnalyseurSyntaxique::lire(){
	al->lire();
	if(al->voirMotCle("PA")){
		pg.genererPA();
		lire();
	}
}

void AnalyseurSyntaxique::lireMotCle(string v){
	al->lireMotCle(v);
	if(al->voirMotCle("PA")){
		pg.genererPA();
		lire();
	}
}


void AnalyseurSyntaxique::lireIdentificateur(){
	al->lireIdentificateur();
	if(al->voirMotCle("PA")){
		pg.genererPA();
		lire();
	}	
}
void AnalyseurSyntaxique::lireCaractere(string v){
	al->lireCaractere(v);
	if(al->voirMotCle("PA")){
		pg.genererPA();
		lire();
	}
}





#endif
