/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./highlight.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


 /****************************************************************************
 **
 ** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights.  These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/


#include <QtGui>
#include "highlight.h"

Highlighter::Highlighter(QTextDocument *parent) : QSyntaxHighlighter(parent){
    HighlightingRule rule;

    formatMC0.setForeground(QColor(0, 153, 255));
    formatMC0.setFontWeight(QFont::Bold);
    
    formatMC1.setFontWeight(QFont::Bold);
    
    formatNombre.setForeground(QColor("blue"));
    formatMC2.setForeground(QColor("blue"));
    
		formatPA.setFontWeight(QFont::Bold);
    formatPA.setForeground(QColor("red"));
    
    QStringList patternsMC0;
    QStringList patternsMC1;
    QStringList patternsMC2;
		QStringList patternsPA;
    
    patternsMC0 << "\\bprogram\\b" << "\\bbegin\\b" << "\\bend\\b";
    patternsMC2 << "\\binteger\\b" << "\\bbool\\b"<<"\\bprint\\b"<<"\\bvoid\\b";
    patternsMC1 <<"\\broutine\\b"<<"\\bwhile\\b"<< "\\bfor\\b"<<"\\bdo\\b"<<"\\bif\\b"<<"\\belse\\b"<<"\\belsif\\b"<<"\\bfi\\b"<<"\\bthen\\b"<<"\\bdone\\b"<<"\\breturn\\b";
		patternsPA <<"\\bPA\\b";
    foreach (const QString &pattern, patternsMC0) {
        rule.pattern = QRegExp(pattern);
        rule.format = formatMC0;
        highlightingRules.append(rule);
    }

    foreach (const QString &pattern, patternsMC1) {
        rule.pattern = QRegExp(pattern);
        rule.format = formatMC1;
        highlightingRules.append(rule);
    }

    foreach (const QString &pattern, patternsMC2) {
        rule.pattern = QRegExp(pattern);
        rule.format = formatMC2;
        highlightingRules.append(rule);
    }
		
		foreach (const QString &pattern, patternsPA) {
			rule.pattern = QRegExp(pattern);
			rule.format = formatPA;
			highlightingRules.append(rule);
    }

    formatComm.setForeground(QColor("green"));
    rule.pattern = QRegExp("//[^\n]*");
    rule.format = formatComm;
    highlightingRules.append(rule);
    
    rule.pattern = QRegExp("\\d+");
    rule.format = formatNombre;
    highlightingRules.append(rule);
    
/*    formatData.setForeground(QColor("red"));
    rule.pattern = QRegExp("end\\..+");
    rule.format = formatData;
    highlightingRules.append(rule);*/
}

void Highlighter::highlightBlock(const QString &text)
{
    foreach (const HighlightingRule &rule, highlightingRules) {
        QRegExp expression(rule.pattern);
        int index = expression.indexIn(text);
        while (index >= 0) {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);
            index = expression.indexIn(text, index + length);
        }
    }
   setCurrentBlockState(0);
}

