/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./interpreteur.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "interpreteur.h"

Interpreteur::Interpreteur(){
  co=1;	
  int i=0;
  while(i<MAXPILE){
    pile[i++] = -1;
  }
}
Interpreteur::~Interpreteur(){
	po.clear();
}

void Interpreteur::newPO(vector<vector<int> > v){
	po = v;
	codeOp = po[0][0];
	co=1;
}

void Interpreteur::executer(){
  int nIp=0;
	int i= MAXPILE-1;
  ostringstream oss;
  string out="";
	switch(codeOp){
		case DEBUT :
			ip=-1;
			base=-1;
		break;
		
		case RESERVER :
			++ip;
			nIp = ip + po[co++][1];
			if(ip>MAXPILE){
				string err = "Debordement de la pile";
				throw(err);
			}
			while(ip<nIp){
				pile[ip++]=-12;
				emit signal_modifierPile(MAXPILE-ip,"NULL",2);
			}
			--ip;
		break;

		case EMPILER :
			pile[++ip] = po[co][1];
			if(po[co][2] == 1){
				emit signal_modifierPile(MAXPILE-1-ip,pile[ip],1); //noir gras
				emit signal_tracerFleche(MAXPILE-1-ip+1,MAXPILE-pile[ip]);
			}else{
				emit signal_modifierPile(MAXPILE-1-ip,pile[ip],0);//noir
			}
			++co;
		break;
		
		case EMPILER_AS :
			pile[++ip] = po[co][1]+ base+1;
			emit signal_modifierPile(MAXPILE-1-ip,pile[ip],1); //noir gras
			emit signal_tracerFleche(MAXPILE-1-ip+1,MAXPILE-pile[ip]);
			++co;
		break;

		case VALPILE :
			pile[ip] = pile[pile[ip]];
			emit signal_modifierPile(MAXPILE-1-ip,pile[ip],0);//noir
			++co;
		break;
		
		case INVERSER :
			pile[ip] = -1 * pile[ip];
			emit signal_modifierPile(MAXPILE-ip,pile[ip],0);//noir
			++co;
		break;
		
		case AFFECTATION :
			pile[pile[ip-1]]=pile[ip];
			emit signal_modifierPile(MAXPILE-1-pile[ip-1],pile[ip],0);//noir
			emit signal_clearLigne(MAXPILE-1-ip+1);
			emit signal_clearLigne(MAXPILE-1-ip);
			ip-=2;
			co+=1;
		break;

		case ADD :
			pile[ip-1]=pile[ip-1]+pile[ip];
			emit signal_clearLigne(MAXPILE-1-ip);
			emit signal_modifierPile(MAXPILE-1-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;

		case SUB :
			pile[ip-1]=pile[ip-1]-pile[ip];
			emit signal_clearLigne(MAXPILE-1-ip);
			emit signal_modifierPile(MAXPILE-1-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;

		case MULT :
			pile[ip-1]=pile[ip-1]*pile[ip];
			emit signal_clearLigne(MAXPILE-1-ip);
			emit signal_modifierPile(MAXPILE-1-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;
		
		case DIV :
			if (pile[ip]==0){
				string err = "Erreur a l'execution : Division par zero";
				throw (err);
			}else{
				pile[ip-1]=pile[ip-1]/pile[ip];
				emit signal_clearLigne(MAXPILE-1-ip);
				emit signal_modifierPile(MAXPILE-1-ip+1,pile[ip-1],0);
				++co;
				--ip;
			}
		break;

		case PRINT :
			out="<Sortie interperteur> : ";
			oss<<pile[ip];
			out+=oss.str();oss.str("");
			emit signal_afficher(out);
			emit signal_clearLigne(MAXPILE-1-ip);
			--ip;
			++co;
		break;
		
		case BZE :
			if(pile[ip] == 0){
				co = po[co][1];
			}else{
				++co;
			}
			emit signal_clearLigne(MAXPILE-1-ip);
			--ip;			
		break;
		
		case BR :
			co = po[co][1];
		break;
		
		case EGAL :
			if(pile[ip] == pile[ip-1]){
				pile[ip-1] = 1;
			}else{
				pile[ip-1] = 0;
			}
			emit signal_clearLigne(MAXPILE-ip);
			emit signal_modifierPile(MAXPILE-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;

		case DIFF :
			if(pile[ip-1] == pile[ip-1]){
				pile[ip-1] = 0;
			}else{
				pile[ip] = 1;
			}
			emit signal_clearLigne(MAXPILE-ip);
			--ip;
			++co;
		break;
		
		case SUP :
			if(pile[ip] < pile[ip-1]){
				pile[ip-1] = 1;
			}else{
				pile[ip-1] = 0;
			}
			emit signal_clearLigne(MAXPILE-ip);
			emit signal_modifierPile(MAXPILE-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;
		
		case SUPE :
			if(pile[ip] <= pile[ip-1]){
				pile[ip-1] = 1;
			}else{
				pile[ip-1] = 0;
			}
			emit signal_clearLigne(MAXPILE-ip);
			emit signal_modifierPile(MAXPILE-ip+1,pile[ip-1],0);
			++co;
			--ip;
		break;
		
		case INF :
			if(pile[ip] > pile[ip-1]){
				pile[ip-1] = 1;
			}else{
				pile[ip-1] = 0;
			}
			emit signal_clearLigne(MAXPILE-ip);
			emit signal_modifierPile(MAXPILE-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;
		
		case INFE :
			if(pile[ip] >= pile[ip-1]){
				pile[ip-1] = 1;
			}else{
				pile[ip-1] = 0;
			}
			emit signal_clearLigne(MAXPILE-ip);
			emit signal_modifierPile(MAXPILE-ip+1,pile[ip-1],0);
			--ip;
			++co;
		break;
		
		case BASE :
			++ip;
			emit signal_modifierPile(MAXPILE-1-ip,"CO",4); //bleu italique
			++ip;
			emit signal_modifierPile(MAXPILE-1-ip,"BASE",4); //bleu italique
			++co;
		break;
		
		case REMPLIR :
			pile[ip -po[co][1]] = base; // code de retour
			emit signal_modifierPile(MAXPILE-1-(ip -po[co][1]),base,4); //bleu italique
			pile[ip -po[co][1] -1] = co+2; // emplacement base appelant
			emit signal_modifierPile(MAXPILE-1-(ip -po[co][1]-1),co+2,4); //bleu italique
			emit signal_tracerFlecheG(MAXPILE-1-(ip -po[co][1]),MAXPILE-base);
			base = ip -po[co][1]; 
			emit signal_bougerBase(MAXPILE-1-base);
			++co;
		break;
		
		case DEROUTER :
			co = po[co][1];
		break;
		
		case RETURN :
			i= pile[ip];
			co = pile[base-1];
      std::cout << co<<endl;
			while(ip>(base-2)){
				emit signal_clearLigne(MAXPILE-1-ip);
				--ip;
			}
			pile[++ip] = i;
			emit signal_modifierPile(MAXPILE-1-ip,i,0);
			base = pile[base];
			emit signal_bougerBase(MAXPILE-base);		
		break;
		
		case REROUTER :
			co = pile[base-1];
			while(ip>(base-2)){
				emit signal_clearLigne(MAXPILE-1-ip);
				--ip;
			}
			base = pile[base];
			emit signal_bougerBase(MAXPILE-base);
		break;
		
		case PA :
			emit signal_PA();
			++co;
		break;
		
		case FIN :
			codeOp = po[0][0];
			co=1;
			throw(1);
		break;
				
		default :
			throw "INSTRUCTION INCORRECTE";
	}
    codeOp=po[co][0];
		emit signal_bougerCO(co);
    emit signal_bougerIP(MAXPILE-1-ip);
}
