/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./pointGeneration.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef PGC
#define PGC
#include "pointGeneration.h"
#include <sstream>
#include <iostream>
#include <QFont>
#include <QBrush>



Generation::Generation(AnalyseurLexical* a) : QObject(){
  al = a;
  addrGlo=0;
  addrLoc=0;
  addr = &addrGlo;
  co =0;
  tdi = &tdiGlobale;
	varGlo_finBloc =0;
	
}
Generation::~Generation(){
	delete addr;
	po.clear();
}

void Generation::TDILocale(){
  tdi = &tdiLocale;
  addr = &addrLoc;
  addrLoc = 0;
  (*tdi).clear();
}

void Generation::TDIGlobale(){
  tdi = &tdiGlobale;
}


void Generation::reset(){
	addrGlo=0;
  addrLoc=0;
	co =0;
  po.clear();
	tdiGlobale.clear();
	tdiLocale.clear();
	tdi = &tdiGlobale;
}

int Generation::typage(string g,string d){
  if(g.compare(d)==0){
    return 1;
  }else{
    return 0;
  }
}

int Generation::typage(string g,string d,string x){
  if((g.compare(d)==0) && (g.compare(x)==0) ){
    return 1;
  }else{
    return 0;
  }
}

string Generation::typeDe(string m){
	if(tdiLocale.estDeclaree(m)){ // variable locale masque globale
		return tdiLocale.typeDe(m);
	}
	return tdiGlobale.typeDe(m);
}

void Generation::savIdent(string m){
  listeIdent.insert(listeIdent.end(),m);
}


void Generation::insererIdents(string type){
  list<string>::iterator i;
  int res=0;
  for(i=listeIdent.begin();i!=listeIdent.end();++i){
    ++res;
    if(!(*tdi).estDeclaree(*i)){
      (*tdi).ajouterVar(*i,type,(*addr)++);
    }else{
      (*al).erreur(9,*i);
    }
  }
  genererReserver(res);
}

void Generation::newDecla(){
  listeIdent.erase(listeIdent.begin(),listeIdent.end());
}

string Generation::genre(string m){
  return (*tdi).genre(m);
}

int Generation::estDeclaree(string m){
	if(tdi == &tdiGlobale){
		if(tdiGlobale.estDeclaree(m)){
			return 1;
		}
	}else{ // Dans une fonction, il faut vérifier que la var est bien declaree en loc ou glob
		if(tdiLocale.estDeclaree(m)){
			return 2;
		}else if(tdiGlobale.estDeclaree(m)){
			return 1;
		}
	}
	return 0;
}


int Generation::estVariable(string m){
	if(tdiLocale.estDeclaree(m)){ // variable locale masque globale
		return tdiLocale.estVariable(m);
	}
	return tdiGlobale.estVariable(m);
}

int Generation::estRoutine(string m){
  return tdiGlobale.estRoutine(m);
}

int Generation::addrVar(string m){
	if(tdiGlobale.estDeclaree(m)){
		return tdiGlobale.addrVar(m);
	}
	return tdiLocale.addrVar(m);

}

void Generation::ajouterParam(string r,string n,string t){ //routine nom type
	tdiLocale.ajouterVar(n,t,addrLoc++,1);
	tdiGlobale.ajouterParam(r,t);
}

void Generation::insererRoutine(string nom,string ret){
	tdiGlobale.ajouterRoutine(nom,ret,co);
}

int Generation::nombreParam(string r){
	return tdiGlobale.nombreParam(r);
}

string Generation::typeParam(string r,int i){
	return tdiGlobale.typeParam(r,i);
}

int Generation::use(string v){
	if(tdiLocale.estDeclaree(v)){ // variable locale masque globale
		return tdiLocale.use(v);
	}
	return tdiGlobale.use(v);
}

int Generation::init(string v){
	if(tdiLocale.estDeclaree(v)){ // variable locale masque globale
		return tdiLocale.init(v);
	}
	return tdiGlobale.init(v);
}

void Generation::estInit(string v){
	if(tdiLocale.estDeclaree(v)){ // variable locale masque globale
		tdiLocale.estInitialisee(v);
	}
	tdiGlobale.estInitialisee(v);	
}

void Generation::estUtilisee(string v){
	if(tdiLocale.estDeclaree(v)){ // variable locale masque globale
		tdiLocale.estUtilisee(v);
	}
	tdiGlobale.estUtilisee(v);	
}

///////////////////////
//
//  Generation de code
//
///////////////////////
void Generation::setFinBloc(int i){
	varGlo_finBloc =i;
}

void Generation::genererDebut(){
  genererI(DEBUT);
}

void Generation::genererFin(){
  genererI(FIN);
}

void Generation::genererEmpiler(int i){
  genererI(EMPILER,i,0);
}

void Generation::genererEmpiler_ADDR(int i){
		genererI(EMPILER,i,1);
}

void Generation::genererEmpiler_ADDR_S(int i){
	genererI(EMPILER_AS,i,2);
}

void Generation::genererInverser(string s){
  if(s.compare("-")==0){
    genererI(INVERSER);
  }
}

void Generation::genererValPile(){
  genererI(VALPILE);
}

void Generation::genererReserver(int i){
  genererI(RESERVER,i);
}

void Generation::genererAdd(string op){
  if(op.compare("+")==0){
    genererI(ADD);
  }else{
    genererI(SUB);
  }
}

void Generation::genererMult(string op){
  if(op.compare("*")==0){
    genererI(MULT);
  }else{
    genererI(DIV);
  }
}

void Generation::genererAnd(){
  genererI(ADD);
}

void Generation::genererOr(){
  genererI(OR);
}

void Generation::genererAffect(){
  genererI(AFFECTATION);
}

void Generation::genererPrint(){
  genererI(PRINT);
}

int Generation::genererBze(){
  genererI(BZE,-1);
  return co-1;
}

int Generation::genererBr(int bze){
  genererI(BR,-1);
  brancher(bze,co);// instr bze doit pointer sous le br
  return co-1;
}

int Generation::genererBr(){
  genererI(BR,-1);
  return co-1;
}

void Generation::brancher(int c){
  if(po[c][0] == BZE || po[c][0] == BR){
    po[c][1] =co; 
    std::ostringstream oss;
    string instr = instrs.instrs(po[c][0]);
    oss <<" "<<co;
    instr+=oss.str();oss.str("");
    emit signal_brancher(instr,c,co);
  }else{
    cout <<"BRANCHEMENT INTERDIT"<<endl;
		throw "BRANCHEMENT INTERDIT";
  }
}

void Generation::brancher(int c,int b){
  if(po[c][0] == BZE || po[c][0] == BR){
    po[c][1] =b;
    std::ostringstream oss;
    string instr = instrs.instrs(po[c][0]);
    oss <<" "<<b;
    instr+=oss.str();oss.str("");
    emit signal_brancher(instr,c,b);
  }else{
    cout <<"BRANCHEMENT INTERDIT : "<<po[c][0]<<endl;
		throw "BRANCHEMENT INTERDIT";
  }
}

void Generation::genererTest(string t){
  if(t.compare("==")==0){
    genererI(EGAL);
  }else if(t.compare("!=")==0){
    genererI(DIFF);
  }else if(t.compare(">")==0){
    genererI(SUP);
  }else if(t.compare(">=")==0){
    genererI(SUPE);
  }else if(t.compare("<")==0){
    genererI(INF);
  }else if(t.compare("<=")==0){
    genererI(INFE);
  }
}


void Generation::genererBase(){
  genererI(BASE);
}

void Generation::remplirBase(int p){
	genererI(REMPLIR,p);
}

void Generation::derouter(string routine){
	genererI(DEROUTER,tdiGlobale.addrVar(routine));
}

void Generation::sortirRoutine(int i){
	if(i==0){
		genererI(REROUTER);
	}else if(i==1){
		genererI(RETURN);
	}
}

void Generation::genererPA(){
	genererI(28);
}

void Generation::genererI(int i){
	if(varGlo_finBloc==0){
		++co;
		vector<int> tmp;
		tmp.push_back(i);
		po.push_back(tmp);
		//std::cout<<"instr sav :"<<i<<" tab : "<<po[po.size()-1][0]<<endl;
		emit signal_newInstr(instrs.instrs(i));
	}
}

void Generation::genererI(int i,int j){
	if(varGlo_finBloc==0){
		++co;
		vector<int> tmp;
		tmp.push_back(i);
		tmp.push_back(j);
		po.push_back(tmp);
		
		std::ostringstream oss;
		string instr = instrs.instrs(i);
		oss <<" "<<j;
		instr+=oss.str();oss.str("");
		emit signal_newInstr(instr);
	}
}


void Generation::genererI(int i,int j,int k){
	if(varGlo_finBloc==0){
		++co;
		vector<int> tmp;
		tmp.push_back(i);
		tmp.push_back(j);
		tmp.push_back(k);
		po.push_back(tmp);
		
		std::ostringstream oss;
		string instr = instrs.instrs(i);
		oss <<" "<<j;
		instr+=oss.str();oss.str("");
		emit signal_newInstr(instr);
	}
	

}



vector<vector<int> > Generation::programmeObjet(){
  return po;
}

void Generation::afficherPO(){
  std::ostringstream oss;
  string ts;
  vector<vector<int> >::iterator i;
  for(i=po.begin();i!=po.end();++i){
    ts = instrs.instrs((*i)[0]);
    if((*i).size()>1){
      oss <<" "<<(*i)[1];
      ts+=oss.str();oss.str("");
    }
  }
}


Element::Element(int a,string t,string g){
    addr =a;
    type = t;
    genre = g;
		use = 0;
		init = 0;
}
Element::~Element(){
	params.clear();
}

TDI::~TDI(){
	table.clear();
	listeTmp.clear();
}

void TDI::ajouterVar(string nom,string type,int addr){
  Element v(addr,type,"variable");
  table.insert(pair<string,Element>(nom,v));
}

void TDI::ajouterVar(string nom,string type,int addr,int use){
  Element v(addr,type,"variable");
  table.insert(pair<string,Element>(nom,v));
	if(use==1){
		estInitialisee(nom);
	}
}

void TDI::ajouterRoutine(string nom,string type,int addr){
  Element v(addr,type,"routine");
  table.insert(pair<string,Element>(nom,v));
}

string TDI::genre(string m){
  if(!estDeclaree(m)) return "NONE";
  return ((*(table.find(m))).second).genre;
}

string TDI::typeDe(string m){
  return ((*(table.find(m))).second).type;
}

int TDI::estDeclaree(string m){
  return (table.find(m) != table.end());
}

int TDI::addrVar(string m){
  return ((*(table.find(m))).second).addr;
}

int TDI::estVariable(string m){
  if(((*(table.find(m))).second).genre.compare("variable")==0){
    return 1;
  }
  return 0;
}

int TDI::estRoutine(string m){
  if(((*(table.find(m))).second).genre.compare("routine")==0){
    return 1;
  }
  return 0;
}

int TDI::nombreParam(string r){
	return ((*(table.find(r))).second).params.size();
}

string TDI::typeParam(string r,int p){
	return ((*(table.find(r))).second).params[p];
}

void TDI::ajouterParam(string r,string t){
	((*(table.find(r))).second).params.push_back(t);
}

void TDI::clear(){
  table.clear();
}

int TDI::use(string v){
	return ((*(table.find(v))).second).use;
}

int TDI::init(string v){
	return ((*(table.find(v))).second).init;
}

void TDI::estInitialisee(string v){
	
	((*(table.find(v))).second).init = 1;
}

void TDI::estUtilisee(string v){
	((*(table.find(v))).second).use = 1;
}

#endif
