/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./pointGeneration.h and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef PGH
#define PGH


#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <QTextEdit>
#include "analyseurLexical.h"
#include "constantes.h"

#include "interpreteur.h"

class Element{
private:
public:
  Element(int,string,string);
	~Element();
  int addr;
  string type;
  string genre;
	vector<string> params;
	int use;
	int init;
};

class TDI{
private:
  map<string,Element> table;
  list<string> listeTmp;
public:
	~TDI();
  void ajouterVar(string,string,int);//nom type addr
	void ajouterVar(string,string,int,int);//nom type addr init (pour les params)
	void ajouterRoutine(string,string,int);//nom type addr
	void ajouterParam(string,string);
  string genre(string);
  string typeDe(string);
  int estDeclaree(string);
  int estVariable(string);
  int estRoutine(string);
  string tdiToString();
  int addrVar(string);
  void clear();
	int nombreParam(string);
	string typeParam(string,int);
	void estUtilisee(string);
	void estInitialisee(string);
	int use(string);
	int init(string);
};



class Generation : public QObject{
  Q_OBJECT
private:
	int varGlo_finBloc;  // permet de savoir d'eviter le code mort
  vector<vector<int> > po;
  TDI tdiGlobale;
  TDI tdiLocale;
  TDI* tdi;
  int* addr;
  int addrGlo;
  int addrLoc;
  list<string> listeIdent;
  AnalyseurLexical* al;
  void genererI(int); void genererI(int,int);void genererI(int,int,int);
  ListeInstrs instrs;
  
  
  
public:
  Generation(AnalyseurLexical*);
	~Generation();
  vector<vector<int> >programmeObjet();
  int co;
  void reset();
  ///////////////////////
  //
  //  Utilisation de la TDI
  //
  ///////////////////////
  
  void insererIdents(string);
	void insererRoutine(string,string);
	void enregistrerParams(string);
  void savIdent(string);
  void ajouterParam(string,string,string);  
  void newDecla();
  string typeDe(string);
  string genre(string);
  int estDeclaree(string);
  int addrVar(string);
  int estVariable(string);
  int estRoutine(string);
	int nombreParam(string);
	string typeParam(string,int);
  
  void TDILocale();
  void TDIGlobale();
  
  ///////////////////////
  //
  //  Points de controle
  //
  ///////////////////////
  int typage(string,string);
  int typage(string,string,string);
  string poToString();
  void afficherPO();
	int init(string);
	int use(string);
	void estUtilisee(string);
	void estInit(string);
  
  ///////////////////////
  //
  //  Generation de code
  //
  ///////////////////////
	void setFinBloc(int);
  void genererDebut();
  void genererFin();
  void genererEmpiler(int);
  void genererEmpiler_ADDR(int);
	void genererEmpiler_ADDR_S(int);
  void genererInverser(string);
  void genererValPile();
  void genererReserver(int);
  void genererAdd(string);
  void genererMult(string);
  void genererAnd();
  void genererOr();
  void genererAffect();
  void genererPrint();
  void genererTest(string);
  int genererBze();
  int genererBr(int);
  int genererBr();
  void genererBase();
	void remplirBase(int);
  void brancher(int);
  void brancher(int,int);
	void derouter(string);
	void sortirRoutine(int);
	
	void genererPA();
signals:
  void signal_newInstr(std::string);
  void signal_brancher(std::string,int,int);
};


#endif
