/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./progcible.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "progcible.h"

ProgCible::ProgCible(QWidget* w) : QGraphicsView(w){
  scene = new QGraphicsScene();
  l = 80;
  h = 30;
  cx = 0;
  cy = 10;
  nbElem=0;
  offset=0;
  setScene(scene);
  setSceneRect(0,0,l+20,0);
  setGeometry(-l,0,l+120,400);

  co = new QGraphicsSimpleTextItem("co >");
  QFont f("System",10,-1,1);
  f.setBold(1);
  co->setFont(f);
  co->setBrush(QBrush(QColor(0, 153, 255),Qt::SolidPattern));
  co->setPos(QPointF(cx-4*8-3,0));
  scene->addItem(co);
}
ProgCible::~ProgCible(){
  scene->clear();
  delete scene;
  delete co;
}

void ProgCible::modifier(int pos,std::string val){
  clearLigne(pos);
  QGraphicsSimpleTextItem* texte = new QGraphicsSimpleTextItem();
  texte->setText(QString(val.c_str()));
  texte->setPos(QPointF(cx,pos*h+cy+h/2-6));
  texte->setZValue(1);
  scene->addItem(texte);
}

void ProgCible::ajouter(std::string val){
  QGraphicsSimpleTextItem* texte = new QGraphicsSimpleTextItem();
  texte->setText(QString(val.c_str()));
  texte->setPos(QPointF(cx/*cx+l/2-6*val.length()*/,nbElem*h+cy+h/2-6));
  texte->setZValue(1);
  scene->addItem(texte);
  scene->addLine(cx,nbElem*h+cy,cx+l,nbElem*h+cy,
  QPen(Qt::gray, 0, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
  nbElem++;
  setSceneRect(0,0,l+20,h*(10+nbElem));
}

void ProgCible::clearLigne(int c){
  removeContenu(c);
  removeFleche(c);
  removeFlecheG(c);
}

void ProgCible::clear(){
  int i = 0;
  while(i<nbElem){
    clearLigne(i++);
  }
  nbElem = 0;
}

void ProgCible::removeContenu(int pos){
  QList<QGraphicsItem *>list =  scene->items(QPoint(cx+3,pos*h+cy+h/2-6));
  QList<QGraphicsItem *>::iterator it = list.begin();
  if((*it)->zValue()!=0){
    delete *it;
  }
}

void ProgCible::removeFleche(int pos){
  QGraphicsItem* item =  scene->itemAt(cx+l+2,pos*h+cy+h/2);
  if(item!=0){
    delete item;
    offset-=5;
  }
}

void ProgCible::removeFlecheG(int pos){
  QGraphicsItem* item =  scene->itemAt(cx-2,pos*h+cy+h/2);
  if(item!=0){
    delete item;
    offset-=5;
  }
}

void ProgCible::tracerFleche(int a,int b){
 QPainterPath* fleche = new QPainterPath();
 QPolygonF trace,bout;
 trace << QPoint(cx+l,a*h+cy+h/2)<< QPoint(cx+l+10+offset,a*h+cy+h/2) <<QPoint(cx+l+10+offset,b*h+cy+h/2) << QPoint(cx+l+7,b*h+cy+h/2);
 bout << QPoint(cx+l+2,b*h+cy+h/2) << QPoint(cx+l+7,b*h+cy+h/2-5) << QPoint(cx+l+7,b*h+cy+h/2+5) <<QPoint(cx+l+2,b*h+cy+h/2);

 fleche->addPolygon(trace);
 fleche->addPolygon(bout);
 scene->addPath(*fleche,QPen(Qt::green, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
 offset+=10;
 delete fleche;
}

void ProgCible::tracerFlecheG(int a,int b){
 QPainterPath* fleche = new QPainterPath();
 QPolygonF trace,bout;
 trace << QPoint(cx,a*h+cy+h/2)<< QPoint(cx-10-offsetG,a*h+cy+h/2) <<QPoint(cx-10-offsetG,b*h+cy+h/2) << QPoint(cx-7,b*h+cy+h/2);
 bout << QPoint(cx-2,b*h+cy+h/2) << QPoint(cx-7,b*h+cy+h/2-5) << QPoint(cx-7,b*h+cy+h/2+5) <<QPoint(cx-2,b*h+cy+h/2);

 fleche->addPolygon(trace);
 fleche->addPolygon(bout);
 scene->addPath(*fleche,QPen(Qt::green, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
 offsetG+=10;
 delete fleche;
}

void ProgCible::bougerCo(int y){
  co->setPos(co->x(),y*h+h/2);
}
