/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./stack.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "stack.h"

Stack::Stack(QWidget* w,int nbCell) : QGraphicsView(w){
  scene = new QGraphicsScene();
  cells = nbCell;
  l = 120;
  h = 30;
  x = 0;
  y = 10;
  offset = 0;
  offsetG =0;
  setScene(scene);
  setSceneRect(0,0,l+20,h*(cells+3)); // +3 cell pour la "base" sous pile
  setGeometry(-l,0,l+200,400);
  draw(nbCell);

  QFont f("System",8,-1,1);
  f.setBold(1);

  ip = new QGraphicsSimpleTextItem("ip >");
  ip->setFont(f);
  ip->setBrush(QBrush(QColor(0, 153, 255),Qt::SolidPattern));
  ip->setPos(QPointF(-60,30*cells+15));


  base = new QGraphicsSimpleTextItem("base >");
  base->setFont(f);
  base->setBrush(QBrush(QColor(0, 255, 255),Qt::SolidPattern));
  base->setPos(QPointF(-60,30*(cells+1)+15));

  scene->addItem(ip);
  scene->addItem(base);
  this->verticalScrollBar()->setValue(this->verticalScrollBar()->maximum());

}
Stack::~Stack(){
  scene->clear();
  delete scene;
  delete base;
  delete ip;
}

void Stack::draw(int c){

  QPen pen(Qt::gray, 0, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin);
  int i=0;
  while(i<c){
  scene->addRect(QRect(x,y+h*i,l,h),pen);
  ++i;
  }
  scene->addRect(QRect(x,y+h*i,l,h),QPen(Qt::gray, 0, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin),QBrush(Qt::gray,Qt::FDiagPattern));
  scene->addRect(QRect(x,y+h*(i+1),l,h),QPen(Qt::gray, 0, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin),QBrush(Qt::gray,Qt::BDiagPattern));
}

void Stack::removeContenu(int pos){
  //QList<QGraphicsItem *>list =  scene->items(QPoint(x+l/2,pos*h+y+h/3));
  QList<QGraphicsItem *>list =  scene->items(QRectF(QPointF(x+2,pos*h+y+2),QPointF(x+l-2,pos*h+y+h-2)));
  QList<QGraphicsItem *>::iterator it = list.begin();
  if((*it)->zValue()!=0){
    delete *it;
  }
}

void Stack::removeFleche(int pos){
  QGraphicsItem* item =  scene->itemAt(x+l+2,pos*h+y+h/2);
  if(item!=0){
    delete item;
    offset-=5;
  }
}

void Stack::removeFlecheG(int pos){
  QGraphicsItem* item =  scene->itemAt(x-2,pos*h+y+h/2);
  if(item!=0){
    delete item;
    offsetG-=5;
  }
}

void Stack::modifier(int pos,int val,int font){
  std::string s="";
  std::ostringstream oss;
  oss<<val;
  s+=oss.str();oss.str("");
  modifier(pos,s,font);
}

void Stack::modifier(int pos,std::string val,int font){
  clearLigne(pos);
  QGraphicsSimpleTextItem* texte = new QGraphicsSimpleTextItem();
  texte->setText(QString(val.c_str()));
  texte->setPos(QPointF(x+l/2-6*val.length(),pos*h+y+h/3));
  if(font==1){//noir gras (adresse)
    texte->setBrush(QBrush(QColor(Qt::black),Qt::SolidPattern));
    QFont f("System",12,-1,0);
    f.setBold(1);
    texte->setFont(f);
  }else if(font==2){//gris italique (null)
    texte->setBrush(QBrush(QColor(Qt::gray),Qt::SolidPattern));
    QFont f("System",12,-1,1);
    texte->setFont(f);
  }else if(font == 3){//gris valeur d'une variable
    texte->setBrush(QBrush(QColor(Qt::gray),Qt::SolidPattern));
    QFont("System",12,-1,0);
  }else if(font==4){//blue italique (base)
    texte->setBrush(QBrush(QColor(Qt::blue),Qt::SolidPattern));
    QFont("System",12,-1,1);
  }else{
    texte->setBrush(QBrush(QColor(Qt::black),Qt::SolidPattern));
    texte->setFont(QFont("System",12,-1,0));
  }
  texte->setZValue(1);
  scene->addItem(texte);
}

void Stack::tracerFleche(int a,int b){
 QPainterPath* fleche = new QPainterPath();
 QPolygonF trace,bout;
 trace << QPoint(x+l,a*h+y-h/2)<< QPoint(x+l+10+offset,a*h+y-h/2) <<QPoint(x+l+10+offset,b*h+y-h/2) << QPoint(x+l+7,b*h+y-h/2);
 bout << QPoint(x+l+2,b*h+y-h/2) << QPoint(x+l+7,b*h+y-h/2-5) << QPoint(x+l+7,b*h+y-h/2+5) <<QPoint(x+l+2,b*h+y-h/2);

 fleche->addPolygon(trace);
 fleche->addPolygon(bout);
 scene->addPath(*fleche,QPen(Qt::red, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
 offset+=5;
 delete fleche;
}

void Stack::tracerFlecheG(int a,int b){
 QPainterPath* fleche = new QPainterPath();
 QPolygonF trace,bout;
 trace << QPoint(x,a*h+y+h/2)<< QPoint(x-10-offsetG,a*h+y+h/2) <<QPoint(x-10-offsetG,b*h+y+h/2) << QPoint(x-7,b*h+y+h/2);
 bout << QPoint(x-2,b*h+y+h/2) << QPoint(x-7,b*h+y+h/2-5) << QPoint(x-7,b*h+y+h/2+5) <<QPoint(x-2,b*h+y+h/2);

 fleche->addPolygon(trace);
 fleche->addPolygon(bout);
 scene->addPath(*fleche,QPen(Qt::green, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
 offsetG+=5;
 delete fleche;
}

void Stack::clearLigne(int c){
    removeContenu(c);
    removeFleche(c);
    removeFlecheG(c);
}

void Stack::clear(){
  int i = 0;
  while(i<cells){
    clearLigne(i++);
  }
  ip->setPos(QPointF(x-29,30*cells+15));
}

void Stack::bougerIp(int y){
  ip->setPos(ip->x(),y*h+15);
}

void Stack::bougerBase(int y){
  base->setPos(ip->x(),y*h+15);
}

