/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./threads.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "threads.h"

Compilateur::Compilateur() : QThread(){
	operation =0;
	semaphore = new QSemaphore(0);
  qRegisterMetaType<std::string>("std::string");
	al = new AnalyseurLexical();
	as = new AnalyseurSyntaxique(al,0,semaphore);
	inter = new Interpreteur();
	connect(al,SIGNAL(signal_newLexeme(std::string)),this,SLOT(slot_newLexeme(std::string)));
	connect(al,SIGNAL(signal_warning(std::string)),this,SLOT(slot_warning(std::string)));
  connect(&(as->pg),SIGNAL(signal_newInstr(std::string)),this,SLOT(slot_newInstr(std::string)));
  connect(&(as->pg),SIGNAL(signal_brancher(std::string,int,int)),
	this,SLOT(slot_brancher(std::string,int,int)));
	setTerminationEnabled(true);

	connect(inter,SIGNAL(signal_modifierPile(int,int,int)),this,SLOT(slot_modifierPile(int,int,int)));
	connect(inter,SIGNAL(signal_modifierPile(int,std::string,int)),this,SLOT(slot_modifierPile(int,std::string,int)));
	connect(inter,SIGNAL(signal_clearLigne(int)),this,SLOT(slot_clearLigne(int)));
	connect(inter,SIGNAL(signal_tracerFleche(int,int)),this,SLOT(slot_tracerFleche(int,int)));
	connect(inter,SIGNAL(signal_tracerFlecheG(int,int)),this,SLOT(slot_tracerFlecheG(int,int)));
	connect(inter,SIGNAL(signal_afficher(std::string)),this,SLOT(slot_afficher(std::string)));
	connect(inter,SIGNAL(signal_bougerCO(int)),this,SLOT(slot_bougerCO(int)));
	connect(inter,SIGNAL(signal_bougerIP(int)),this,SLOT(slot_bougerIP(int)));
	connect(inter,SIGNAL(signal_PA()),this,SLOT(slot_PA_inter()));
}
Compilateur::~Compilateur(){
	delete al;
	delete as;
	delete inter;
	delete semaphore;
}

void Compilateur::run(){
// 	std::cout << "Demarrage du thread\n";
	if(operation==0){
		al->analyser(src);
		emit signal_fin_AL();
	}else if(operation ==1){
		try{
			as->analyser();    
			inter->newPO(as->progObjet());
			emit signal_fin_AS();
		}catch (Erreur err){
			emit signal_erreur_AS(err.numLigne(),err.numColonne(),err.what());
		}
	}else if(operation == 2){
		try{
			inter->executer();
		}catch(std::string err){
			inter->newPO(as->progObjet());
			emit signal_erreur_inter(err);
		}catch(int i){
			if(i ==1){
				emit signal_fin_I();
			}
		}
	}
	//reset semaphore. Au cas ou l'arret fut violent
	while(semaphore->available()){
		semaphore->acquire(1);
	}
}

void Compilateur::slot_lancer_AL(std::string t){
	cout << "DEMANDE ANALYSE LEXICALE"<<endl;
  src = t;
	operation=0;
}

void Compilateur::slot_lancer_AS(){
	cout << "DEMANDE ANALYSE SYNTAXIQUE"<<endl;
	operation=1;
}

void Compilateur::slot_progresser_AL(){
  //sAL->release(1);
}

void Compilateur::slot_pap_AS(){
	operation = 1;
	as->setPasApas();
}

void Compilateur::slot_progresser_AS(){
  semaphore->release(1);
}

void Compilateur::slot_newLexeme(std::string t){
  emit signal_newLexeme(t);
}

void Compilateur::slot_newInstr(std::string t){
  emit signal_newInstr(t);
}

void Compilateur::slot_findLexeme(int i){
  emit signal_resultFindLexeme(al->ligne(i),al->colonne(i));
}

void Compilateur::slot_brancher(std::string t,int i,int j){
  emit signal_brancher(t,i,j);
}

vector<vector<int> > Compilateur::progObjet(){
	return as->progObjet();
}

void Compilateur::slot_warning(std::string s){
	emit signal_warning(s);
}

void Compilateur::executer(){
	cout<<"DEMANDE INTERPRETATION\n";
	operation=2;
}

void Compilateur::slot_modifierPile(int i,int j,int k){
	emit signal_modifierPile(i,j,k);
}
void Compilateur::slot_modifierPile(int i,std::string s,int k){
	emit signal_modifierPile(i,s,k);
}
void Compilateur::slot_clearLigne(int i){
	emit signal_clearLigne(i);
}
void Compilateur::slot_tracerFleche(int i,int j){
	emit signal_tracerFleche(i,j);
}
void Compilateur::slot_tracerFlecheG(int i,int j){
	emit signal_tracerFlecheG(i,j);
}
void Compilateur::slot_afficher(std::string t){
	emit signal_afficher(t);
}
void Compilateur::slot_bougerCO(int i){
	emit signal_bougerCO(i);
}
void Compilateur::slot_bougerIP(int i){
	emit signal_bougerIP(i);
}
void Compilateur::slot_PA_inter(){
	cout<<"Point d'arret\n";
	emit signal_PA_inter();
}
void Compilateur::slot_PA_inter_ACK(){
	semaphore->release(1);
}