/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./threads.h and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <QSemaphore>
#include <QThread>
#include <string>
#include "analyseurLexical.h"
#include "analyseurSyntaxique.h"
#include "interpreteur.h"
#include "exceptions.h"

#include "stack.h"
#include "infos.h"
#include "progcible.h"

class Compilateur : public QThread{
  Q_OBJECT
private:
  QSemaphore* semaphore;
  std::string src;
  
  AnalyseurLexical* al;
  AnalyseurSyntaxique* as;
	Interpreteur * inter;
	
  Stack* stack;
  Infos* infos;
	ProgCible* progCible;
  
	int operation;
  int operationAL;	//variables d'etat pour savoir ou en est le thread.
  int operationAS;
public:
	Compilateur();
	~Compilateur();
  void run();
	/*void analyseSyntaxique();
	void analyseLexicale();*/
	vector<vector<int> > progObjet();
	void executer();
	
public slots:
	
  void slot_lancer_AL(std::string);
  void slot_progresser_AL();
  void slot_lancer_AS();
  void slot_progresser_AS();
  
  void slot_newLexeme(std::string);
  void slot_newInstr(std::string);
  void slot_findLexeme(int);
  void slot_brancher(std::string,int,int);
	void slot_pap_AS();
	
	void slot_warning(std::string);
	
	
	void slot_modifierPile(int,int,int); //case valeur police
	void slot_modifierPile(int,std::string,int); //case valeur police
	void slot_clearLigne(int);
	void slot_tracerFleche(int,int);
	void slot_tracerFlecheG(int,int);
	void slot_afficher(std::string);
	void slot_bougerIP(int);
	void slot_bougerCO(int);	
	void slot_PA_inter();
	void slot_PA_inter_ACK();
	
	
	/*
	 *	SIGNAUX VERS L'INTERFACE
	*/
signals:
	//Signaux envoyés par l'analyseur lexicale
  void signal_newLexeme(std::string);
  void signal_resultFindLexeme(int,int);
	void signal_fin_AL();

	//Signaux envoyés par l'analyseur syntaxique
	void signal_newInstr(std::string);
	void signal_brancher(std::string,int,int);
	void signal_warning(std::string);
	void signal_erreur_AS(int,int,std::string);
	void signal_fin_AS();
	
	
	
	//Signaux envoyés par l'interpreteur
	void signal_modifierPile(int,int,int); //case valeur police
	void signal_modifierPile(int,std::string,int);
	void signal_clearLigne(int);
	void signal_tracerFleche(int,int);
	void signal_tracerFlecheG(int,int);
	void signal_afficher(std::string);
	void signal_bougerCO(int);
	void signal_bougerIP(int);
	void signal_PA_inter();
	void signal_fin_I();
	void signal_erreur_inter(std::string);
};
