/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./windows.cpp and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "windows.h"
#include <QGridLayout>


GUI::GUI():QWidget(){
  
  ///////////////////////////////////
  //
  //	Mise en forme de la fenetre
  //
  ///////////////////////////////////
  
  QPalette cpalette = palette();
  QBrush brush(QColor(230, 230, 230, 255));//gris clair
  brush.setStyle(Qt::SolidPattern);
  cpalette.setBrush(QPalette::Active, QPalette::Window, brush);
  this->setPalette(cpalette);
  QWidget::setWindowTitle("Ktopsy");
  
  
  ///////////////////////////////////
  //
  //	Creation des elements
  //
  ///////////////////////////////////
  
   Notes* notes = new Notes();
	 //notes->open();
  
  boutons = new Boutons(this);
  //zones de texte
  text = new CodeEditor(this);
  infos = new Infos(this);
  stack = new Stack(this,MAXPILE);
  liste = new Liste(this);
  progCible = new ProgCible(this);
	
	compilateur = new Compilateur();
  connect(this,SIGNAL(signal_lancer_AL(std::string)),compilateur,SLOT(slot_lancer_AL(std::string)));
  connect(compilateur,SIGNAL(signal_newLexeme(std::string)),this,SLOT(slot_newLexeme(std::string)));
  connect(compilateur,SIGNAL(signal_fin_AL()),this,SLOT(slot_fin_AL()));
	connect(compilateur,SIGNAL(signal_warning(std::string)),this,SLOT(slot_warning_AS(std::string)));
 	
  
  connect(this,SIGNAL(signal_lancer_AS()),compilateur,SLOT(slot_lancer_AS()));
  connect(compilateur,SIGNAL(signal_newInstr(std::string)),this,SLOT(slot_newInstr(std::string)));
  connect(compilateur,SIGNAL(signal_fin_AS()),this,SLOT(slot_fin_AS()));
  connect(compilateur,SIGNAL(signal_erreur_AS(int,int,std::string)),this,SLOT(slot_erreur_AS(int,int,std::string)));
  
  connect(this,SIGNAL(signal_findLexeme(int)),compilateur,SLOT(slot_findLexeme(int)));
  connect(compilateur,SIGNAL(signal_resultFindLexeme(int,int)),this,SLOT(slot_resultFindLexeme(int,int)));
  connect(compilateur,SIGNAL(signal_brancher(std::string,int,int)),this,SLOT(slot_brancher(std::string,int,int)));
  
  
	connect(compilateur,SIGNAL(signal_fin_I()),this,SLOT(slot_fin_I()));
	connect(compilateur,SIGNAL(signal_modifierPile(int,int,int)),this,SLOT(slot_modifierPile(int,int,int)));
	connect(compilateur,SIGNAL(signal_modifierPile(int,std::string,int)),this,SLOT(slot_modifierPile(int,std::string,int)));
	connect(compilateur,SIGNAL(signal_clearLigne(int)),this,SLOT(slot_clearLigne(int)));
	connect(compilateur,SIGNAL(signal_tracerFleche(int,int)),this,SLOT(slot_tracerFleche(int,int)));
	connect(compilateur,SIGNAL(signal_tracerFlecheG(int,int)),this,SLOT(slot_tracerFlecheG(int,int)));
	connect(compilateur,SIGNAL(signal_afficher(std::string)),this,SLOT(slot_afficher(std::string)));
	connect(compilateur,SIGNAL(signal_bougerCO(int)),this,SLOT(slot_bougerCO(int)));
	connect(compilateur,SIGNAL(signal_bougerIP(int)),this,SLOT(slot_bougerIP(int)));
	connect(compilateur,SIGNAL(signal_PA_inter()),this,SLOT(slot_PA_inter()));
	connect(compilateur,SIGNAL(signal_erreur_inter(std::string)),this,SLOT(slot_erreur_inter(std::string)));
	
	connect(this,SIGNAL(signal_THE_END()),notes,SLOT(close()));
	connect(this,SIGNAL(signal_THE_END()),text,SLOT(close()));
	  
  ///////////////////////////////////
  //
  //	Positionnement des elements
  //
  ///////////////////////////////////
  progCible->hide();
  (*text).setGeometry(10,(*boutons).y()+(*boutons).height()+10,500,400); //X,Y,l,h
  (*progCible).move((*text).width()+(*text).x()+10,(*text).y());
  (*liste).setGeometry(progCible->x(),progCible->y(),progCible->width(),progCible->height());
  (*stack).move((*progCible).width()+(*progCible).x()+10,(*progCible).y());
  (*infos).setGeometry(10,(*text).height()+(*text).y()+10,(*text).width(),100);
      
//   (*text).setPlainText("program:\ninteger i;\nbegin\ni=0;\nwhile i<10 do\nprint(i);\ni=i+1\ndone\nend.");
//   (*text).setPlainText("program:\ninteger i;\nbegin\ni=0;\nif i<10 then\nprint(i)\nelse\nprint(12)\nfi\nend.");
// (*text).setPlainText("program:\ninteger i;\nbegin\nfor i=0;i<10;i=i+1 then\nprint(i)\ndone\nend.");
//(*text).setPlainText("program:\ninteger i;\nroutine void test(integer k)\ninteger j;\nbegin\nj=k*2;\nprint(j)\nend;\nbegin\ni=12;\ntest(i)\nend.");
	(*text).setPlainText("program:\ninteger i;\nroutine void essai(integer k)\nbegin\nk=k+1;\nprint(k)\nend;\nroutine void test(integer k)\ninteger j;\nbegin\nj=k*2;\nprint(j);\nessai(j)\nend;\nbegin\ni=12;\ntest(i)\nend.");
  

  connect(liste, SIGNAL(clicked(QModelIndex)), this, SLOT(lexemeSelect()));
  //timer pour l'execution en continu
  timer = new QTimer(this);
  connect(timer,SIGNAL(timeout()),this,SLOT(executer()));
  ///////////////////////////////////
  //
  //  Connexion des elements aux slots
  //  ( La deconnexion sert a desactiver les boutons )
  //
  ///////////////////////////////////
  
  boutons->desactiver("aS");
  boutons->desactiver("execution");
  boutons->desactiver("pap");
  boutons->desactiver("stop");
  
  ///////////////////////////////////
  //
  //  init variables et objets
  //
  ///////////////////////////////////
  lastSelectedLexeme = -1;
  
}
GUI::~GUI(){
	delete boutons;
	delete liste;
	delete stack;
	delete progCible;
	delete infos;
	delete compilateur;
	delete notes;
}

/*
 *  slot qui remet a zero l'application
*/
void GUI::reset(){
  liste->clear();
  progCible->clear();

  liste->show();
  progCible->hide();
  infos->setText("");
  stack->clear();
	
	boutons->desactiver("pap");
	boutons->desactiver("execution");
	boutons->desactiver("aS");
}


/*
 *  slot qui lance l'analyse lexicale en continu
*/
void GUI::slot_lancer_AL(){
	text->desurligner();
	liste->clear();
	liste->show();
  progCible->hide();
	emit signal_lancer_AL(((*text).toPlainText()).toStdString());
	compilateur->start();
}


/*
 *  slot qui lance l'analyse syntaxique en continu
*/
void GUI::slot_lancer_AS(){
	progCible->clear();
	emit signal_lancer_AS();
	compilateur->start();
  liste->hide();
  progCible->show();
}

void GUI::execRapide(){
  interpreter(0);
}

void GUI::execNormal(){
  interpreter(500);
}

void GUI::execLent(){
  interpreter(1500);
}

/*
 *  slot permettant de lancer l'execution en continu (lance le timer)
 */
void GUI::interpreter(int ms){
  boutons->desactiver("reset");
  boutons->activer("stop");
  timer->start(ms);
}


/*
 *  slot permettant de surligner le texte correspondant le lexeme selectionne dans la liste
*/
void GUI::lexemeSelect(){  
  //recuperer le lexeme selectionne dans la liste
  QModelIndexList q = (*((*liste).selectionModel())).selectedIndexes();
  text->desurligner();

  //si il s'agit de la selection d'un nouveau lexeme alors le surligner sinon le desurligner
  if(lastSelectedLexeme != (q[0]).row()){
    lastSelectedLexeme = (q[0]).row();
    emit signal_findLexeme((q[0]).row());
  }
  
}


/*
 *  slot permettant d'arreter manuellement l'execution (bouton stop)
*/
void GUI::stopInter(){
  timer->stop();
  (*infos).afficherWarning("execution arretee manuellement");
  boutons->desactiver("stop");
	boutons->activer("reset");
}

void GUI::slot_PA_inter(){
  timer->stop();
  (*infos).afficherWarning("execution arretee par un point d'arret");
	emit signal_PA_inter_ACK();
}


/*
 *  slot se chargeant de l'execution en continu (lance par le timer)
 */
void GUI::executer(){
		 compilateur->executer();
		 compilateur->start();		 
}

/*
*	slot lancant une seule et unique instruction
*/
void GUI::slot_pap_inter(){
	if(timer->isActive()){
		timer->stop();
	}
	try{
		compilateur->executer();
		compilateur->start();
	}catch(string err){
    timer->stop();
    (*infos).afficherErreur(err);
    boutons->activer("reset");
    boutons->desactiver("pap");
	}catch(int f){
		if(f==1){
      (*infos).afficherInfo("execution terminee");
    }
		boutons->desactiver("pap");
	}
}


void GUI::slot_fin_AL(){
  (*infos).afficherInfo("analyse lexicale terminee");
	boutons->activer("aS");
}

void GUI::slot_fin_AS(){
  (*infos).afficherInfo("analyse syntaxique terminee");
	boutons->desactiver("aS");
	boutons->activer("execution");
	boutons->activer("pap");
}

void GUI::slot_newLexeme(std::string t){
  liste->remplir(t);
}

void GUI::slot_newInstr(std::string t){
  progCible->ajouter(t);
}

void GUI::slot_brancher(std::string t,int i,int j){
  progCible->modifier(i,t);
  progCible->tracerFleche(i,j);
}

void GUI::slot_resultFindLexeme(int l,int c){
  text->surlignerMot(l,c);
}

void GUI::slot_erreur_AS(int l,int c,std::string t){
  infos->afficherErreur(t);
	text->surlignerErreur(l,c);
}

void GUI::slot_warning_AS(std::string t){
  infos->afficherWarning(t);
}

void GUI::slot_fin_I(){
	timer->stop();
	boutons->activer("reset");
	boutons->desactiver("stop");
	infos->afficherInfo("Execution terminee");
}
void GUI::slot_erreur_inter(std::string v){
	timer->stop();
	boutons->activer("reset");
	boutons->desactiver("stop");
	infos->afficherErreur(v);
}
void GUI::slot_modifierPile(int i,int j,int k){
	stack->modifier(i,j,k);
}
void GUI::slot_modifierPile(int i,std::string v,int k){
	stack->modifier(i,v,k);
}
void GUI::slot_clearLigne(int i){
	stack->clearLigne(i);
}
void GUI::slot_tracerFleche(int i,int j){
	stack->tracerFleche(i,j);
}
void GUI::slot_tracerFlecheG(int i,int j){
	stack->tracerFlecheG(i,j);
}
void GUI::slot_afficher(std::string t){
	infos->afficherSortie(t);
}
void GUI::slot_bougerCO(int i){
	progCible->bougerCo(i);
}
void GUI::slot_bougerIP(int i){
	stack->bougerIp(i);
}

void GUI::closeEvent(QCloseEvent* e){
	emit signal_THE_END(); // Un signal pour eviter le segmentation fault...
	e->accept();
}
