/*
 * Copyright (C) 2010 Dimitry Ernot <dimitry.ernot@gmail.com>
 *
 * This file ./windows.h and program is part of Ktopsy.
 *
 * Ktopsy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *
 * Ktopsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ktopsy.  If not, see <http://www.gnu.org/licenses/>.
 */



#include <QTimer>

#include "constantes.h"
#include "codeeditor.h"

#include <iostream>
//#include "analyseurLexical.h"
//#include "analyseurSyntaxique.h"
#include "interpreteur.h"
#include "threads.h"

#include "boutons.h"
#include "infos.h"
#include "liste.h"
#include "notes.h"
#include "progcible.h"
#include "stack.h"




/*****************************************
*
*		fenetre permettant de compiler et d'executer
*		du code ecrit en pig. 1er onglet
*
*		permet de lancer l'analyse lexicale & syntaxique puis
*		l'execution (pas a pas ou non).
*
*		- une zone de texte avec coloration syntaxique
*		- une zone de texte affichant les informations importantes
*		- une liste affichant la chaine codee apres l'analyse lexicale
*			puis le code objet apres l'analyse syntaxique
*		- une table representant la pile. (utilisee lors de l'execution
*			uniquement)
*****************************************/
class GUI : public QWidget{
  Q_OBJECT
private:
  
  Compilateur* compilateur;
  Interpreteur* inter;
  
  Boutons* boutons;
  CodeEditor* text;
  Infos* infos;
  Stack* stack;
  Liste* liste;  
  ProgCible* progCible;
	Notes* notes;
  QTimer* timer;
  
  //autres variables
  int lastSelectedLexeme;
  int papCommence;
public:
  GUI();
	~GUI();
  void interpreter(int);
  void closeEvent(QCloseEvent*);
  
public slots:
  void reset();
  void lexemeSelect();
  void stopInter();
  void executer();
  
  void execRapide();
  void execNormal();
  void execLent();
 
  void slot_lancer_AL();
  void slot_fin_AL();
	void slot_newLexeme(std::string);
  
  void slot_lancer_AS();
  void slot_fin_AS();
  void slot_erreur_AS(int,int,std::string);
	void slot_warning_AS(std::string);
  
  
  void slot_newInstr(std::string);
  
  void slot_resultFindLexeme(int,int);
  void slot_brancher(std::string,int,int);
	
	void slot_pap_inter();
	void slot_fin_I();
	void slot_modifierPile(int,int,int); //case valeur police
	void slot_modifierPile(int,std::string,int); //case valeur police
	void slot_clearLigne(int);
	void slot_tracerFleche(int,int);
	void slot_tracerFlecheG(int,int);
	void slot_afficher(std::string);
	void slot_bougerCO(int);
	void slot_bougerIP(int);
	void slot_PA_inter();
	void slot_erreur_inter(std::string);
	
	
	
signals:
  void signal_lancer_AL(std::string);
  void signal_lancer_AS();
  void signal_continuer_AS();
  void signal_findLexeme(int);
	void signal_demandeArret();
	void signal_PA_inter_ACK();
	
	void signal_THE_END();
};

